from transformers import PretrainedConfig
import torch.nn as nn

class AIIAConfig(PretrainedConfig):
    model_type = "AIIA"  # Add this class attribute

    def __init__(
        self,
        kernel_size: int = 3,
        activation_function: str = "GELU",
        hidden_size: int = 512,
        num_hidden_layers: int = 12,
        num_channels: int = 3,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.kernel_size = kernel_size
        self.activation_function = activation_function
        self.hidden_size = hidden_size
        self.num_hidden_layers = num_hidden_layers
        self.num_channels = num_channels

        # Store additional keyword arguments as attributes
        for key, value in kwargs.items():
            setattr(self, key, value)

    @property
    def activation_function(self):
        return self._activation_function

    @activation_function.setter
    def activation_function(self, value):
        attr = getattr(nn, value, None)
        if attr is None or (not callable(attr) and not isinstance(attr, type(nn.Module))):
            valid_funcs = [func for func in dir(nn) if callable(getattr(nn, func)) or isinstance(getattr(nn, func), type(nn.Module))]
            raise ValueError(f"Invalid activation function: {value}. Choose from: {', '.join(valid_funcs)}")
        self._activation_function = value

    def to_dict(self):
        # Recursively converts the object's attributes into serializable Python types.
        def serialize(value):
            if hasattr(value, "to_dict"):
                return value.to_dict()
            elif isinstance(value, list):
                return [serialize(item) for item in value]
            elif isinstance(value, dict):
                return {k: serialize(v) for k, v in value.items()}
            return value
        return {k: serialize(v) for k, v in self.__dict__.items()}